/*
 * Copyright (c) CERN 2013-2017
 *
 * Copyright (c) Members of the EMI Collaboration. 2011-2013
 *  See  http://www.eu-emi.eu/partners for details on the copyright
 *  holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <apr_strings.h>
#include <ctype.h>
#include <dmlite/c/catalog.h>
#include <dmlite/c/checksums.h>
#include <httpd.h>
#include <regex.h>
#include <shared/utils.h>
#include <http_log.h>
#include "mod_lcgdm_ns.h"


dav_error* dav_ns_digest_header(request_rec *r, const dav_resource *resource,
        char* output, size_t outsize)
{
    const char *want_digest;
    char digest_name[32], digest[64], full_digest_name[64];
    int has_digest;

    want_digest = apr_table_get(r->headers_in, "Want-Digest");

    if (!want_digest) {
        return NULL;
    }

    has_digest = 0;
    while (dav_shared_next_digest(&want_digest, digest_name, sizeof(digest_name))) {
        snprintf(full_digest_name, sizeof(full_digest_name), "checksum.%s", digest_name);

        int ret = dmlite_getchecksum(resource->info->ctx,
            resource->info->sfn, // logical name
            full_digest_name, digest, sizeof(digest),
            NULL, // replica
            0, // 0 = do not recalculate
            5 // 5 seconds wait, otherwise return EAGAIN
            );

        if (ret == 0 && digest[0] != '\0') {
            size_t len;
            len = snprintf(output, outsize, "%s=%s,", digest_name, digest);
            output += len;
            outsize -= len;
            has_digest = 1;
        } else if (DMLITE_ERRNO(dmlite_errno(resource->info->ctx)) == EAGAIN) {
            return dav_shared_new_error(resource->info->request, resource->info->ctx,
                202, "Checksum not available yet. Try again later.");
        } else if (dmlite_errno(resource->info->ctx) != 0) {
            ap_log_rerror(APLOG_MARK, APLOG_WARNING, 0, resource->info->request,
                "Failed to get the checksum %s: %s", digest_name, dmlite_error(resource->info->ctx));
        } else {
            ap_log_rerror(APLOG_MARK, APLOG_WARNING, 0, resource->info->request,
                "Failed to get the checksum %s: empty value", digest_name);
        }
    }

    if (*(output-1) == ',') {
        --output;
    }
    *output = '\0';

    return NULL;
}
